// package dein.package;

import java.io.*;


/**
 * Klassenname: Sylk<BR>
 *<BR>
 * Subsystem: Frontend Export Excel<BR>
 *<BR>
 * Zugeh&ouml;riger UseCase: Excel-Export Posten sowie Summen- und Zinsberechnung Posten<BR>
 *<BR>
 * Erzeugen einer SYLK-Datei ( oder Byte-Array fuer Servlet-Response ) als Br&uuml;cke zum Excel-Export.<BR>
 * Zum SYLK-Format siehe Buch Addison-Wesley "Dateiformate"<BR>
 * Beim Einstellen des MIME-Types "application/vnd.ms-excel" ( Excel ) erkennt Excel
 * das SYLK-Format. Der Browser &ouml;ffnet Excel ( wenn installiert ).<BR>
 * In Excel kann die HTTP-Response angesehen und editiert werden, sowie &uuml;ber
 * die Excel-Men&uuml;-Leiste abgespeichert werden.<BR>
 * Es werden nicht alle SYLK-Features unterst&uuml;tzt.<BR>
 * Das zu erzeugende Excel-Sheet wird in einem zweidimensionalen Array von SylkCell-Objekten zwischengespeichert.
 * Durch entsprechende setCell...-Methoden k&ouml;nnen Werte sowie Formeln in den Zellen gesetzt werden.
 * Mit der Methode write kann das aufgebaute Sheet dann in einen OutputStream gesendet werden.<BR>
 * <BR>
 * Alle Koordinaten f&uuml;r Zellenzeilen- und Zellenspaltenpositionen werden mit der Basis 1 angegeben.<BR>
 * <BR>
 * In dieser Version ist es m&ouml;glich, &uuml;ber den Schalter bClEraseMode zu erreichen,<BR>
 * da&szlig; beim Aufbau der SYLK-Response nicht mehr ben&ouml;tigte Daten gel&ouml;scht werden.<BR>
 * Beim Debugging mit Hilfe der Methode toFile mu&szlig; bClEraseMode ausgeschaltet werden.<BR>
 * <BR>
 * Es wird die Subklasse SylkCell verwendet.<BR>
 *<BR>
 * Die Klasse Sylk wird in den Klassen PostenExportExcel, PostenSummeExportExcel und PostenZinsExportExcel benutzt.
 *
 * @author Heiner K&uuml;cker
 * @version $Id$
 *
 */
public class Sylk {

  /**
   * zweidimensionales Puffer-String-Array
   */
  private SylkCell[][] sylkCellArrCl;

  /**
   * Array zum Speichern der Spaltenbreiten
   */
  private int[] iArrClColumnWidth;

  /**
   *<BR>
   * Schalter, ob nicht mehr ben&ouml;tigte Zellen-Daten gel&ouml;scht werden d&uuml;rfen<BR>
   * Default-Einstellung ist true
   */
  private boolean bClEraseMode = true;

  /**
   * Merker, da&szlig; die Spaltenbreite auf Default steht
   */
  public static final int COLUMN_WIDTH_DEFAULT = -1;

  /**
   *<BR>
   * Konstruktor<BR>
   * erstellen eines neuen Sylk-Objects mit entsprechender Gr&ouml;sse<BR>
   * Dimensionen: 1 ... 65536 Zeilen , 1 ... 256 Spalten<BR>
   * bei Nichteinhaltung der Dimensionsgrenzen wird eine Exception geworfen ( siehe Excel-Hilfe Grenzen-Arbeitsumgebung )
   *
   * @param iPaRows Anzahl der Zeilen
   * @param iPaColumns Anzahl der Spalten
   *
   * @throws Exception
   */
  public Sylk(int iPaRows, int iPaColumns)
      throws Exception {
    if (iPaRows < 1 && iPaRows > 65536 && iPaColumns < 1 && iPaColumns > 256) {
      throw new Exception("Sylk-Dimension nicht zulaessig");
    }

    sylkCellArrCl = new SylkCell[ iPaRows ][ iPaColumns ];

    iArrClColumnWidth = new int[ iPaRows ];

    // beide Arrays initialisieren
    for (int i = 0; i < iPaRows; i++) {
      for (int j = 0; j < iPaColumns; j++) {
        sylkCellArrCl[ i ][ j ] = new SylkCell();
      }

      iArrClColumnWidth[ i ] = COLUMN_WIDTH_DEFAULT;
    }

  } // end Konstruktor

  /**
   *<BR>
   * Abfragen L&ouml;schmode-Klassenvariable bClEraseMode,
   * welche bestimmt, ob nicht mehr ben&ouml;tigte Zellen-Daten gel&ouml;scht werden d&uuml;rfen.
   *
   * @return bClEraseMode
   */
  public boolean getbClEraseMode() {
    return bClEraseMode;
  } // end method getbClEraseMode

  /**
   *<BR>
   * Setzen L&ouml;schmode-Klassenvariable bClEraseMode,
   * welche bestimmt, ob nicht mehr ben&ouml;tigte Zellen-Daten gel&ouml;scht werden d&uuml;rfen.
   *
   * @param zu setzender Wert (logisch ja/nein)
   */
  public void setbClEraseMode(boolean bPaEraseMode) {
    bClEraseMode = bPaEraseMode;
  } // end method setbClEraseMode

  /**
   * Setzen eines Zellwertes mit einem String<BR>
   *<BR>
   * Bei Schreibversuch ausserhalb der Sheet-Gr&ouml;sse oder bei zu langem String wird eine Exception geworfen
   *
   * @param iPaRow      Zeilenkoordinate der Zelle ( Basis 1 )
   * @param iPaColumn   Spaltenkoordinate der Zelle ( Basis 1 )
   * @param strPaValue  in die Zelle einzutragender String ( maximale Laenge 32.000 Zeichen )
   *
   * @throws Exception
   */
  public void setCell(int iPaRow, int iPaColumn, String strPaValue)
      throws Exception {
    String strTmp;

    if (strPaValue != null) {
      if (strPaValue.length() <= 32000) {
        iPaRow--;
        iPaColumn--;
        if (iPaRow >= 0 && iPaRow < sylkCellArrCl.length && iPaColumn >= 0 && iPaColumn < sylkCellArrCl[ iPaRow ].length) {
          strTmp = convertUmlaute(strPaValue);
          sylkCellArrCl[ iPaRow ][ iPaColumn ].setValue("\"" + strTmp + "\"");
        }
        else {
          throw new Exception("Zugriff ausserhalb Sylk-Dimensionen");
        }
      }
      else {
        throw new Exception("Textlaenge " + strPaValue.length() + " nicht zulaessig ( maximal 32.000 )");
      }
    }
  } // end method setCell

  /**
   *<BR>
   * Setzen eines Zellwertes mit einem String und Ausrichtung<BR>
   *<BR>
   * Bei Schreibversuch ausserhalb der Sheet-Gr&ouml;sse oder bei zu langem String wird eine Exception geworfen
   *
   * @param iPaRow       Zeilenkoordinate der Zelle ( Basis 1 )
   * @param iPaColumn    Spaltenkoordinate der Zelle ( Basis 1 )
   * @param strPaValue   in die Zelle einzutragender String ( maximale L&auml;nge 32.000 Zeichen )
   * @param iPaAlignment Ausrichtung der Zelle ( hier bitte die bereitgestellten Konstanten benutzen )
   *
   * @throws Exception
   */
  public void setCell(int iPaRow, int iPaColumn, String strPaValue, int iPaAlignment)
      throws Exception {
    String strTmp;

    if (strPaValue != null) {
      if (strPaValue.length() <= 32000) {
        iPaRow--;
        iPaColumn--;
        if (iPaRow >= 0 && iPaRow < sylkCellArrCl.length && iPaColumn >= 0 && iPaColumn < sylkCellArrCl[ iPaRow ].length) {
          strTmp = convertUmlaute(strPaValue);
          sylkCellArrCl[ iPaRow ][ iPaColumn ].setValue("\"" + strTmp + "\"");
          sylkCellArrCl[ iPaRow ][ iPaColumn ].setAlignment(iPaAlignment);
        }
        else {
          throw new Exception("Zugriff ausserhalb Sylk-Dimensionen");
        }
      }
      else {
        throw new Exception("Textlaenge " + strPaValue.length() + " nicht zulaessig ( maximal 32.000 )");
      }
    }
  } // end method setCell

  /**
   *<BR>
   * Setzen eines Zellwertes mit einem double-Elementartyp<BR>
   *<BR>
   * Bei Schreibversuch ausserhalb der Sheet-Gr&ouml;sse wird eine Exception geworfen
   *
   * @param iPaRow       Zeilenkoordinate der Zelle ( Basis 1 )
   * @param iPaColumn    Spaltenkoordinate der Zelle ( Basis 1 )
   * @param dPaValue     in die Zelle einzutragender double-Wert ( Elementartyp )
   *
   * @throws Exception
   */
  public void setCell(int iPaRow, int iPaColumn, double dPaValue)
      throws Exception {
    iPaRow--;
    iPaColumn--;
    if (iPaRow >= 0 && iPaRow < sylkCellArrCl.length && iPaColumn >= 0 && iPaColumn < sylkCellArrCl[ iPaRow ].length) {
      sylkCellArrCl[ iPaRow ][ iPaColumn ].setValue("" + dPaValue); // eventuell hier die Methode mit String verwenden
    }
    else {
      throw new Exception("Zugriff ausserhalb Sylk-Dimensionen");
    }
  } // end method setCell

  /**
   * Setzen eines Zellwertes mit einem Double-Object<BR>
   *<BR>
   * bei Schreibversuch ausserhalb der Sheet-Gr&ouml;sse oder bei &Uuml;bergabe null-Pointer wird eine Exception geworfen
   *
   * @param iPaRow       Zeilenkoordinate der Zelle ( Basis 1 )
   * @param iPaColumn    Spaltenkoordinate der Zelle ( Basis 1 )
   * @param dblPaValue   in die Zelle einzutragendes Double-Object ( falls ein null-Pointer &uuml;bergeben wird, wirft diese Methode eine Exception )
   *
   * @throws Exception
   */
  public void setCell(int iPaRow, int iPaColumn, Double dblPaValue)
      throws Exception {
    if (dblPaValue != null) {
      setCell(iPaRow, iPaColumn, dblPaValue.doubleValue());
    }
    else {
      throw new Exception("Double-Object is null");
    }
  } // end method setCell

  /**
   * Setzen eines Zellwertes mit einer Formel<BR>
   *<BR>
   * bei Schreibversuch ausserhalb der Sheet-Gr&ouml;sse oder bei zu langem String wird eine Exception geworfen
   *
   * @param iPaRow        Zeilenkoordinate der Zelle ( Basis 1 )
   * @param iPaColumn     Spaltenkoordinate der Zelle ( Basis 1 )
   * @param strPaFormula  in die Zelle einzutragende Formel als String ( maximale L&auml;nge 1024 Zeichen )
   *
   * @throws Exception
   */
  public void setCellFormula(int iPaRow, int iPaColumn, String strPaFormula)
      throws Exception {
    String strTmp;

    iPaRow--;
    iPaColumn--;
    if (strPaFormula != null) {
      if (strPaFormula.length() < 1025) {
        if (iPaRow >= 0 && iPaRow < sylkCellArrCl.length && iPaColumn >= 0 && iPaColumn < sylkCellArrCl[ iPaRow ].length) {
          sylkCellArrCl[ iPaRow ][ iPaColumn ].setValue(convertUmlaute(strPaFormula));
          sylkCellArrCl[ iPaRow ][ iPaColumn ].setStatus(SylkCell.CELL_FORMULA);
        }
        else {
          throw new Exception("Zugriff ausserhalb Sylk-Dimensionen");
        }
      }
      else {
        throw new Exception("Formellaenge " + strPaFormula.length() + " nicht zulaessig ( maximal 1024 )");
      }
    }
  } // end method setCell

  /**
   *<BR>
   * Setzen des Zell-Alignments auf Default ( Strings linksb&uuml;ndig , Zahlen rechtsb&uuml;ndig )<BR>
   *<BR>
   * bei Schreibversuch ausserhalb der Sheet-Gr&ouml;sse wird eine Exception geworfen
   *
   * @param iPaRow     Zeilenkoordinate der Zelle ( Basis 1 )
   * @param iPaColumn  Spaltenkoordinate der Zelle ( Basis 1 )
   *
   * @throws Exception
   */
  public void setCellAlignDefault(int iPaRow, int iPaColumn)
      throws Exception {
    iPaRow--;
    iPaColumn--;
    if (iPaRow >= 0 && iPaRow < sylkCellArrCl.length && iPaColumn >= 0 && iPaColumn < sylkCellArrCl[ iPaRow ].length) {
      sylkCellArrCl[ iPaRow ][ iPaColumn ].setAlignment(SylkCell.ALIGN_DEFAULT);
    }
    else {
      throw new Exception("Zugriff ausserhalb Sylk-Dimensionen");
    }
  } // end method setCell

  /**
   * Setzen des Zell-Alignments auf rechtsb&uuml;ndig<BR>
   *<BR>
   * Bei Schreibversuch ausserhalb der Sheet-Gr&ouml;sse wird eine Exception geworfen
   *
   * @param iPaRow     Zeilenkoordinate der Zelle ( Basis 1 )
   * @param iPaColumn  Spaltenkoordinate der Zelle ( Basis 1 )
   *
   * @throws Exception
   */
  public void setCellAlignRight(int iPaRow, int iPaColumn)
      throws Exception {
    iPaRow--;
    iPaColumn--;
    if (iPaRow >= 0 && iPaRow < sylkCellArrCl.length && iPaColumn >= 0 && iPaColumn < sylkCellArrCl[ iPaRow ].length) {
      sylkCellArrCl[ iPaRow ][ iPaColumn ].setAlignment(SylkCell.ALIGN_RIGHT);
    }
    else {
      throw new Exception("Zugriff ausserhalb Sylk-Dimensionen");
    }
  } // end method setCell

  /**
   * Setzen des Zell-Alignments auf zentriert<BR>
   *<BR>
   * Bei Schreibversuch ausserhalb der Sheet-Gr&ouml;sse wird eine Exception geworfen
   *
   * @param iPaRow     Zeilenkoordinate der Zelle ( Basis 1 )
   * @param iPaColumn  Spaltenkoordinate der Zelle ( Basis 1 )
   *
   * @throws Exception
   */
  public void setCellAlignCenter(int iPaRow, int iPaColumn)
      throws Exception {
    iPaRow--;
    iPaColumn--;
    if (iPaRow >= 0 && iPaRow < sylkCellArrCl.length && iPaColumn >= 0 && iPaColumn < sylkCellArrCl[ iPaRow ].length) {
      sylkCellArrCl[ iPaRow ][ iPaColumn ].setAlignment(SylkCell.ALIGN_CENTER);
    }
    else {
      throw new Exception("Zugriff ausserhalb Sylk-Dimensionen");
    }
  } // end method setCell

  /**
   * Setzen der Spaltenbreite ( Default ist 10 Zeichen )<BR>
   *<BR>
   * bei Schreibversuch ausserhalb der Sheet-Gr&ouml;sse oder unzul&auml;ssiger Spaltenbreite wird eine Exception geworfen
   *
   * @param iPaColumn Spaltenkoordinate ( Basis 1 )
   * @param iPaWidth  einzustellende Spaltenbreite zulaessig ist 0 ... 255
   *
   * @throws Exception
   */
  public void setColumnWidth(int iPaColumn, int iPaWidth)
      throws Exception {
    iPaColumn--;
    if (iPaWidth >= 0 && iPaWidth < 256) // Spaltenbreite abpruefen
    {
      if (iPaColumn >= 0 && iPaColumn < sylkCellArrCl[ 0 ].length) {
        iArrClColumnWidth[ iPaColumn ] = iPaWidth;
      }
      else {
        throw new Exception("Zugriff ausserhalb Sylk-Dimensionen");
      }
    }
    else {
      throw new Exception("Spaltenbreite " + iPaWidth + " nicht zulaessig ( 0 ... 255 )");
    }
  } // end method setCell

  /**
   *<BR>
   * Setzen einer Additionsformel in einer Zelle<BR>
   * Addiert die die Zellen Zelle1+Zelle2 ( nicht von:bis ).<BR>
   * Angegeben werden die Koordinaten der Zelle, in der die Summen stehen soll,<BR>
   * die Koordinaten der ersten Zelle, welche addiert werden soll und<BR>
   * die Koordinaten der zweiten Zelle, welche addiert werden soll.<BR>
   * <BR>
   *
   * bei Schreibversuch ausserhalb der Sheet-Gr&ouml;sse wird eine Exception geworfen<BR>
   *
   * @param int iPaRow      Zeilenposition  der Zelle in der die Summenformel stehen soll
   * @param int iPaColumn   Spaltenposition der Zelle in der die Summenformel stehen soll
   * @param int iPaRow1     Zeilenposition  der ersten  Zelle, welche addiert werden soll
   * @param int iPaColumn1  Spaltenposition der ersten  Zelle, welche addiert werden soll
   * @param int iPaRow2     Zeilenposition  der zweiten Zelle, welche addiert werden soll
   * @param int iPaColumn2  Spaltenposition der zweiten Zelle, welche addiert werden soll
   *
   * @throws Exception
   */
  // herausgenommen, weil es unter Office 2000 nicht funktioniert
  // bei Office 2000 eckige Klammern statt runder: (R(-22)C(-1)+R(-1)C(-1)) -> (R[-22]C[-1]+R[-1]C[-1])
  // public void setCellAddFormel(int iPaRow, int iPaColumn, int iPaRow1, int iPaColumn1, int iPaRow2, int iPaColumn2)
  // throws Exception {
  // // Achtung: die Formel wird in SYLK anders gespeichert als in Excel dargestellt
  // // relativ-Positionierung
  // // (R(-22)C(-1)+R(-1)C(-1))
  // setCellFormula(iPaRow, iPaColumn,
  // "(R(" + (iPaRow1 - iPaRow) + ")C(" + (iPaColumn1 - iPaColumn) + ")+R(" + (iPaRow2 - iPaRow) + ")C("
  // + (iPaColumn2 - iPaColumn) + "))");
  // } // end method setCellAddFormel

  /**
   *<BR>
   * Setzen einer Summenformel in einer Zelle<BR>
   * Angegeben werden die Koordinaten der Zelle, in der die Summenformel stehen soll,<BR>
   * die Koordinaten der Zelle, bei der das Summieren beginnt ( inklusive ) und <BR>
   * die Koordinaten der Zelle, bei der das Summieren endet ( inklusive ).<BR>
   * <BR>
   * bei Schreibversuch ausserhalb der Sheet-Gr&ouml;sse wird eine Exception geworfen
   *
   * @param int iPaRow             Zeilenposition  der Zelle in der die Summenformel stehen soll
   * @param int iPaColumn          Spaltenposition der Zelle in der die Summenformel stehen soll
   * @param int iPaRowFrom         Zeilenposition  der Zelle, bei der das Summieren beginnt ( inklusive )
   * @param int iPaColumnFrom      Spaltenposition der Zelle, bei der das Summieren beginnt ( inklusive )
   * @param int iPaRowTo           Zeilenposition  der Zelle, bei der das Summieren endet ( inklusive )
   * @param int iPaColumnTo        Spaltenposition der Zelle, bei der das Summieren endet ( inklusive )
   *
   * @throws Exception
   */
  // herausgenommen, weil es unter Office 2000 nicht funktioniert
  // bei Office 2000 eckige Klammern statt runder.
  // public void setCellSummeFormel(int iPaRow, int iPaColumn, int iPaRowFrom, int iPaColumnFrom, int iPaRowTo, int iPaColumnTo)
  // throws Exception {
  //
  // // Achtung: die Formel wird in SYLK anders gespeichert als in Excel dargestellt
  // // relativ-Positionierung
  // // SUM(R(-22)C(-1):R(-1)C(-1))
  // setCellFormula(iPaRow, iPaColumn,
  // "SUM(R(" + (iPaRowFrom - iPaRow) + ")C(" + (iPaColumnFrom - iPaColumn) + "):R(" + (iPaRowTo - iPaRow) + ")C("
  // + (iPaColumnTo - iPaColumn) + "))");
  // } // end method setCellSummeFormel

  /**
   *<BR>
   * Setzen einer bedingten Summenformel in einer Zelle<BR>
   * Angegeben werden die Koordinaten der Zelle, in der die Summenformel stehen soll,<BR>
   * die Koordinaten der Zelle, bei der die Abfrage des IF-Kriteriums beginnt ( inklusive ) und <BR>
   * die Koordinaten der Zelle, bei der die Abfrage des IF-Kriteriums endet ( inklusive ),<BR>
   * das Summierkriterium ohne einschlieende Anf&uuml;hrungszeichen,<BR>
   * die Koordinaten der Zelle, bei der das Summieren beginnt ( inklusive ) und <BR>
   * die Koordinaten der Zelle, bei der das Summieren endet ( inklusive ).<BR>
   * <BR>
   * bei Schreibversuch ausserhalb der Sheet-Gr&ouml;sse wird eine Exception geworfen
   *
   * @param int iPaRow             Zeilenposition  der Zelle in der die Summenformel stehen soll
   * @param int iPaColumn          Spaltenposition der Zelle in der die Summenformel stehen soll
   * @param int iPaIfRowFrom       Zeilenposition  der Zelle, bei der das Summieren beginnt ( inklusive )
   * @param int iPaIfColumnFrom    Spaltenposition der Zelle, bei der das Summieren beginnt ( inklusive )
   * @param int iPaIfRowTo         Zeilenposition  der Zelle, bei der das Summieren endet ( inklusive )
   * @param int iPaIfColumnTo      Spaltenposition der Zelle, bei der das Summieren endet ( inklusive )
   * @param String strPaIfItem     IF-Kriterium ( Summierkriterium ) ohne einschlieende Anf&uuml;hrungszeichen
   * @param int iPaRowFrom         Zeilenposition  der Zelle, bei der das Summieren beginnt ( inklusive )
   * @param int iPaColumnFrom      Spaltenposition der Zelle, bei der das Summieren beginnt ( inklusive )
   * @param int iPaRowTo           Zeilenposition  der Zelle, bei der das Summieren endet ( inklusive )
   * @param int iPaColumnTo        Spaltenposition der Zelle, bei der das Summieren endet ( inklusive )
   *
   * @throws Exception
   */
  // herausgenommen, weil es unter Office 2000 nicht funktioniert
  // bei Office 2000 eckige Klammern statt runder.
  // public void setCellSummeWennFormel(int iPaRow, int iPaColumn,
  // int iPaIfRowFrom, int iPaIfColumnFrom, int iPaIfRowTo, int iPaIfColumnTo,
  // String strPaIfItem,
  // int iPaRowFrom, int iPaColumnFrom, int iPaRowTo, int iPaColumnTo)
  // throws Exception {
  // // Achtung: die Formel wird in SYLK anders gespeichert als in Excel dargestellt
  // // relativ-Positionierung
  // // C;Y23;K121;ESUMIF(R(-22)C(-2):R(-1)C(-2),"=offen",R(-22)C:R(-1)C)
  // setCellFormula(iPaRow, iPaColumn,
  // "SUMIF(" + "R(" + (iPaIfRowFrom - iPaRow) + ")C(" + (iPaIfColumnFrom - iPaColumn) + "):R(" + (iPaIfRowTo - iPaRow)
  // + ")C(" + (iPaIfColumnTo - iPaColumn) + ")" + ",\"" + strPaIfItem + "\"," + "R(" + (iPaRowFrom - iPaRow) + ")C("
  // + (iPaColumnFrom - iPaColumn) + "):R(" + (iPaRowTo - iPaRow) + ")C(" + (iPaColumnTo - iPaColumn) + "))");
  //
  // } // end method setCellSummeWennFormel

  /**
   * <BR>
   * konvertiert in einem &uuml;bergebenen String Umlaute, sz, Semikolon und CR+LF ( Enter ) nach SYLK-Konventionen
   *
   * @param strPaValue zu konvertierender String
   *
   * @return konvertierter String
   */
  private String convertUmlaute(String strPaValue) {
    String strTmp = "";
    String strTmpOne;

    if (strPaValue != null) {
      for (int i = 0; i < strPaValue.length(); i++) {
        strTmpOne = strPaValue.substring(i, i + 1);
        if (strTmpOne.equals("")) {
          strTmpOne = "\u001bNHA";
        }
        else if (strTmpOne.equals("")) {
          strTmpOne = "\u001bNHO";
        }
        else if (strTmpOne.equals("")) {
          strTmpOne = "\u001bNHU";
        }
        else if (strTmpOne.equals("")) {
          strTmpOne = "\u001bNHa";
        }
        else if (strTmpOne.equals("")) {
          strTmpOne = "\u001bNHo";
        }
        else if (strTmpOne.equals("")) {
          strTmpOne = "\u001bNHu";
        }
        else if (strTmpOne.equals("")) {
          strTmpOne = "\u001bN{";
        }
        else if (strTmpOne.equals(";")) {
          strTmpOne = ";;";
        } // Kcker 24.04.01
        else if (strTmpOne.equals("\n")) {
          strTmpOne = "\u001b :";
        } // LineFeed
        else if (strTmpOne.equals("\r")) {
          strTmpOne = "\u001b :";
        } // CarriageReturn
        strTmp += strTmpOne;
      }
    }
    else {
      strTmp = null;
    }
    return strTmp;
  } // end method convertUmlaute

  /**
   * Gibt die SYLK-Datei als byte-Array zur&uuml;ck
   *
   * @throws IOException
   */
  public byte[] getBytes()
      throws IOException {
    // geschtzter Platzbedarf je Zelle 100 Zeichen
    ByteArrayOutputStream outPuffer = new ByteArrayOutputStream(sylkCellArrCl.length * sylkCellArrCl[ 0 ].length * 100);

    write(outPuffer);

    return outPuffer.toByteArray();
  } // end method getBytes

  /**
   * schreibt die Sylk-Datei in einen OutputStream
   *
   * @param outputStream in diesen OutputStream ist auszugeben
   *
   * @throws IOException
   */
  public void write(OutputStream outputStream)
      throws IOException {
    int iArrayWidth = sylkCellArrCl[ 0 ].length;

    // ID-Record
    outputStream.write(("ID;EcisPosten.SYLK\n").getBytes());

    // B -> Boundary-Record
    outputStream.write(("B;Y" + sylkCellArrCl.length + ";X" + sylkCellArrCl[ 0 ].length + "\n").getBytes());

    // Schleife ueber Zeilen ( Rows )
    for (int i = 0; i < sylkCellArrCl.length; i++) {
      // Schleife ueber Spalten ( Columns )
      for (int j = 0; j < iArrayWidth; j++) {
        if (sylkCellArrCl[ i ][ j ].getValue() != null) {
          // C -> Cell-Record    K -> Zellwert    E -> Formel
          if (sylkCellArrCl[ i ][ j ].getStatus() == SylkCell.CELL_VALUE) {
            // Zellwert
            outputStream.write(("C;Y" + (i + 1) + ";X" + (j + 1) + ";K" + sylkCellArrCl[ i ][ j ].getValue() + "\n").getBytes());
          }
          else {
            // Zell-Formel
            outputStream.write(("C;Y" + (i + 1) + ";X" + (j + 1) + ";E" + sylkCellArrCl[ i ][ j ].getValue() + "\n").getBytes());
          }

          // Alignment
          // rechts und zentriert
          // Default wird nicht gesetzt
          if (sylkCellArrCl[ i ][ j ].getAlignment() == SylkCell.ALIGN_RIGHT) {
            // rechtsbuendig
            // F -> Format-Record
            outputStream.write(("F;Y" + (i + 1) + ";X" + (j + 1) + ";FG0R\n").getBytes());
          }
          else if (sylkCellArrCl[ i ][ j ].getAlignment() == SylkCell.ALIGN_CENTER) {
            // zentriert
            // F -> Format-Record
            outputStream.write(("F;Y" + (i + 1) + ";X" + (j + 1) + ";FG0C\n").getBytes());
          }
          else if (sylkCellArrCl[ i ][ j ].getAlignment() == SylkCell.ALIGN_LEFT) {
            // zentriert
            // F -> Format-Record
            outputStream.write(("F;Y" + (i + 1) + ";X" + (j + 1) + ";FG0L\n").getBytes());
          }
        }
      } // for

      // Spaltenbreiten
      if (iArrClColumnWidth[ i ] > COLUMN_WIDTH_DEFAULT) {
        // nicht Default

        // F -> Format-Record
        outputStream.write(("F;W" + (i + 1) + " " + (i + 1) + " " + iArrClColumnWidth[ i ] + "\n").getBytes());
      }

      if (bClEraseMode) {
        // Freigeben der ausgegebenen Zeile zur Speicherplatzeinsparung
        sylkCellArrCl[ i ] = null;
      }

    } // for

    // Ende SYLK-File
    outputStream.write(("E\n").getBytes());

  } // end method getBytes

  /**
   *<BR>
   * Speichert das Sylk-Excel-Sheet als Datei<BR>
   * Verwenden Sie die Datei-Namens-Extension   .slk   oder  .xls
   *
   * @param strPaFileName Name und Pfad der Zieldatei als String
   *
   * @throws IOException
   */
  public void toFile(String strPaFileName)
      throws IOException {
    File file = new File(strPaFileName);

    if (!file.exists()) {
      file.createNewFile();
    }
    FileOutputStream fileOutputStream = new FileOutputStream(file);

    // fileOutputStream.write( getBytes() ) ;
    write(fileOutputStream);
    fileOutputStream.close();
  } // end method toFile

  /**
   * Destruktor
   *
   * @throws Throwable
   */
  protected void finalize()
      throws Throwable {
    super.finalize();
  } // end Destruktor

} // end class Sylk
